# Run the 4 separate preamp corrections for HECTOSPEC
# Doug Mink, October 16, 2007
#
# Input may be a comma-separated list or a filename (with prefaced "@")
# If output list not given, the updates are in place, else the output
# list must match the input in number.
#

procedure hpreamp (input)

string input = ""	{prompt="Image to correct"}

bool    deltemp=yes     {prompt="Delete intermediate image files (y or n)"}
bool    saveold=yes     {prompt="Save unprocessed image files (y or n)"}
bool    verbose=yes     {prompt="Print process progress (y or n)"}

begin

file	imroot, temp1, imin, templist, imout
string	imx, ims, imsec, history, tchar, pfix
bool	dellist, vb

   string inlist, outlist
   string tchar
   string pcsect1, psect2, psect3, psect4
   string fname
   file   imagei, imageo

    vb = verbose
    ims  = input
   
   procid = mktemp("tmp$")   # really the procid plus a character
# Make directory in which to save raw multi-extension files
    if (saveold) {
	if (!access ("Raw"))
	    mkdir ("Raw")
	}

#  Make temporary list of input files
    tchar = substr (ims,1,1)
    if (tchar == '@') {
	templist = substr (ims,2,100)
	deltemp = no
	}
    else {
	templist = mktemp ("proclist")
	files (ims, >templist)
	}

    list = templist
    while (fscan(list, imagei) != EOF) {
	if (vb)
	    printf("PREAMPFIX1: fix preamps on extns of %s to %s\n",
		   str(imagei), str(imageo))

# If preamp fix has been done, note it and go on to next image
	imgetp.default = ""
	imgetp (image, "PREAMPFX")
	if (imgetp.value != "") {
	    if (vb) {
		print ("PREAMPFIX already done for ",imagei)
		print ("PREAMPFIX: ",imgetp.value)
		}
	    next
	    }


# Copy image extensions to separate files and process them
	pfsect1 = imroot // "PF_0"
	imcopy (imagei // "[0]", pfsect0, verbose=vb)

	pfsect1 = imroot // "PF_1"
	imcopy (imagei // "[1]", pfsect1, verbose=vb)
	preamp1 (pfsect1)

	pfsect2 = imroot // "PF_2"
	imcopy (imagei // "[2]", pfsect2, verbose=vb)
	preampf1 (pfsect1)

	pfsect3 = imroot // "PF_3"
	imcopy (imagei // "[3]", pfsect3, verbose=vb)
	preampfix1 (pfsect3)

	pfsect4 = imroot // "PF_4"
	imcopy (imagei // "[4]", pfsect4, verbose=vb)
	preampfix1 (pfsect4)

# Save the unchanged image to the Raw/ directory or delete it
	if (sv) {
	    outfile = "Raw/" // imagei
	    rename (imagei, outfile)
	    }
	else
	    delete (imagei)

# Reassemble the multi-extension FITS

     if(vb) print("PREAMPFIX: joining ",imageo)
  # assemble the output file
     mscjoin(procid // "PF", output = imageo, delete=yes, verbose=vb)
   }

   delete(procid // "*")

end

