# TPREPROC - Run image processing tasks on both TRES amps, then combine them
# September 28, 2007
# By Doug Mink, SAO Telescope Data Center

procedure tpreproc (images)

string	images=""	{prompt="Images to merge"}
real	gaincorr=1.0	{prompt="Gain correction for amp2 to amp1"}
bool	fixbias=no	{prompt="Fix bias section (yes or no)"}
bool	deltemp=yes	{prompt="Delete intermediate image files (y or n)"}
bool	saveold=yes	{prompt="Save unprocessed image files (y or n)"}
bool	verbose=yes	{prompt="Print process progress (y or n)"}

begin

file	imroot, amp1, amp2, temp1, temp2, temp3, im2, imin, templist
file	imfile, newfile, imlist
string	chip1, chip2, arg1, imx, ims, imsec, history, tchar, imex
string	binning
bool	vb

	vb = verbose
	imx = ".fits"
	ims = image

	if (ims == "") {
	    print ("TPREPROC: Run image processing tasks one amp at a time, then combine amps")
	    print ("Usage: tpreproc im1,im2,...,imn")
	    print ("Usage: tpreproc *.fits")
	    print ("Usage: tpreproc @image_list_file")
	    return
	    }

	if (!deftask ("ccdproc"))
	    mscred

# Make directory in which to save unprocessed files
	if (saveold) {
	    if (!access ("Raw"))
		mkdir ("Raw")
	    }

#  Make temporary list of input files
	tchar = substr (ims,1,1)
	templist = mktemp ("temproclist")
	if (tchar == '@') {
	    imlist = substr (ims,2,100)
	    copy (imlist, templist)
	    }
	else {
	    files (ims, >templist)
	    }

# Read through input files
	list = templist
        while (fscan (list,imin) != EOF) {

# Do not process image twice
	    imgetp.default = "0"
	    imsec = imin // "[0]"
	    imgetp (imsec, "TPREPROC")
#	    imgetp (imin, "TPREPROC")
	    if (imgetp.value != "0") {
		if (vb) {
		    print ("TPREPROC: ", imin, " has already been processed")
		    print ("TPREPROC = ", imgetp.value)
		    }
		}

# Process image file
	    else {

#  Rename original file
		if (saveold) {
		    imr = "Raw/" // imin
		    if (!access (imr)) {
			if (vb)
			    print ("TPREPROC: Copying ",imin," to Raw/"
			copy (imin, imr)
			}
		    else if (vb)
			print ("TPREPROC: ",imin," is in Raw/ already"
		    }

# Find root name of image file
		imroot = imin
		smatch (imroot, imx)
		if (smatch.found) {
		    imroot = substr (imroot,1,smatch.start-1)
		    }
		else {
		    imroot = imin
		    imin = imin // imx
		    }
#		if (vb) {
#		    print ("TPREPROC: image root is ",imroot)
#		    }

# Specify FITS extensions for the single-preamplifier images
		amp1 = imin // "[1]"
		amp2 = imin // "[2]"

# Find out binning of image
		if (fixbias) {
		    imgetp.default = "0"
		    imgetp (amp1, "CCDSUM")
		    if (imgetp.value != "0") {
			if (imgetp.value == "1 1")
			    binning = "1"
			else
			    binning = "2"
			}
		    else
			binning = "1"

# Fix bias coordinates in single-preamp images for CCDPROC
		    if (binning == "1") {
			hedit (amp1,"BIASSEC","[1075:1090,3:4608]",update+,verify-,show=vb)
			hedit (amp2,"BIASSEC","[1075:1090,3:4608]",update+,verify-,show=vb)
			}
		    else {
			hedit (amp1,"BIASSEC","[541:545,2:2313]",update+,verify-,show=vb)
			hedit (amp2,"BIASSEC","[541:545,2:2313]",update+,verify-,show=vb)
			}
		    }

#  Trim, subtract overscan, and fix bad pixels using imred.ccdred.ccdproc
		ccdproc.bpmasks = ""
		ccdproc.ccdtype = ""
		ccdproc.noproc = no
		ccdproc.xtalkcor = no
		ccdproc.fixpix = no
		ccdproc.overscan = yes
		ccdproc.trim = yes
		ccdproc.zerocor = no
		ccdproc.darkcor = no
		ccdproc.flatcor = no
		ccdproc.sflatcor = no
		ccdproc.split = no
		ccdproc.merge = yes
		ccdproc.xtalkfile = ""
		ccdproc.fixfile = ""
		ccdproc.saturation = "INDEF"
		ccdproc.sgrow = 0
		ccdproc.bleed = "INDEF"
		ccdproc.btrail = 20
		ccdproc.bgrow = 0
		ccdproc.biassec = "!BIASSEC"
		ccdproc.trimsec = "!TRIMSEC"
		ccdproc.zero = "ZERO"
		ccdproc.dark = "DARK"
		ccdproc.flat = "FLAT"
		ccdproc.sflat = ""
		ccdproc.minreplace = 1.
		ccdproc.interactive = no
		ccdproc.function = "legendre"
		ccdproc.order = 1
		ccdproc.sample = "*"
		ccdproc.naverage = 1
		ccdproc.niterate = 5
		ccdproc.low_reject = 3.
		ccdproc.high_reject = 3.
		ccdproc.grow = 0.
		ccdproc.fd = ""
		ccdproc.fd2 = ""

#  Trim, subtract overscan, and fix bad pixels using mscred.ccdproc
		if (vb)
		    print ("TPREPROC: Running ccdproc for fixpix, overscan, and trim")
		temp2 = imroot // "_out.fits"
		ccdproc (imin, output=temp2)
		if (vb)
		    print ("TPREPROC: fixpix, overscan, and trim ", imin," -> ", temp2)
		imdel (imin)
		rename (temp2, imin)

# Set history line including time
		gettime ()
		history = gettime.date // " " // gettime.time // " fixpix, overscan, and trim"

#  Recombine two amplifier files into single image file
		tampmerge.saveold = saveold
		tampmerge.deltemp = deltemp
		tampmerge.verbose = vb
		tampmerge (imin)

# Add history line including time
		hedit (imin, "TPREPROC", history, add+, verify-)
		}
	    }

	if (deltemp)
	    delete (templist)
end

# Sep 28 2007	New program to process list of TRES images
