# File tres/Scripts/treslist.cl
# By Doug Mink
# October 25, 2007

# TRESLIST -- Make Starbase table of major keywords in TRES data files
#             and make lists for FLATs and COMPs

procedure treslist (spectra)

string spectra="\*.fits"	{prompt="Files to process"}

string indir=""		{prompt="Input directory for data files"}
string object=""	{prompt="Select this object if not null"}
string apfib=""		{prompt="Select fiber (s m l) binning (b) ap (1 2 3)"}
file outfile=""		{prompt="File name for output list of files"}
bool addfib=yes		{prompt="Add FIBi keywords to spectra (yes or no)"}
bool deltemp=yes	{prompt="Delete temporary files (yes or no"}
bool verbose=yes	{prompt="Log progress (yes or no)"}
bool debug=no		{prompt="Print debugging information (yes or no)"}

begin
    bool vb, db, dt
    file imf, tempilist
    file listfile, ofile
    string fibkey, fib1, fib2, fib3, af, aff, afb, afs, afx
    int ns
    string specs, outlist
    string filename
    string dline1, dline2
    char tchar

    vb = verbose
    db = debug
    dt = deltemp
    ofile = outfile
    af = apfib
    afx = af
    if (substr(afx,1,1) == "s") {
	afs = "s"
	afx = substr (afx,2,10)
	}
    else if (substr(afx,1,1) == "m") {
	afs = "m"
	afx = substr (afx,2,10)
	}
    else if (substr(afx,1,1) == "l") {
	afs = "l"
	afx = substr (afx,2,10)
	}
    else
	afs = ""
    if (substr (afx,1,1) == "b") {
	afb = "b"
	afx = substr (afx,2,10)
	}
    else
	afb = ""
    if (substr (afx,1,1) == "1") {
	aff = "1"
	afx = substr (afx,2,10)
	}
    else if (substr (afx,1,1) == "2") {
	aff = "2"
	afx = substr (afx,2,10)
	}
    else if (substr (afx,1,1) == "3") {
	aff = "3"
	afx = substr (afx,2,10)
	}
    else
	aff = ""
    specs = spectra

# Make sure output directory is there

#  Make temporary list of input spectrum files
    tempilist = mktemp ("tempilist")
    tchar = substr (specs,1,1)
    if (tchar == '@') {
	listfile = substr (specs,2,100)
	copy (listfile, tempilist)
	}
    else {
	files (specs, >tempilist)
	}
    dline1 = "filename     	type	object	peri	bin	fibsize	ossw	exptime	cmptime	fibkey	fib1	fib2	fib3"
    dline2 = "-----------	-------	------	----	---	-------	----	-------	-------	------	----	----	----"
    
    tresfib.addfib = addfib
    tresfib.verbose = db
    tresfib.debug = db

    ns = 0
    if (vb) {
	print (dline1)
	print (dline2)
	}
    outlist = ""
    list = tempilist
    while (fscan (list, imf) != EOF) {
	filename = imf
	tresfib (filename)
	if (db)
	    dpar ("tresfib")
	fibkey = tresfib.fibkey
	fib1 = tresfib.fib1
	fib2 = tresfib.fib2
	fib3 = tresfib.fib3
	if (db)
	    print ("AF=",af," AFF=",aff," afs=",afs," afb=",afb)
	if (af != "") {
	    if (aff == "1" && fib1 == "OFF")
		next
	    if (aff == "2" && fib2 == "OFF")
		next
	    if (aff == "3" && fib3 == "OFF")
		next
	    if (afb != tresfib.binning)
		next
	    if (afs != "" && afs != tresfib.apsize)
		next
	    }
	if (object != "" && tresfib.object != object)
	    next
	ns = ns + 1
	if (vb)
	    printf ("%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s	%s\n", filename, tresfib.imtype, tresfib.object, tresfib.peri, tresfib.bin1, tresfib.fibsize, tresfib.osswitch, tresfib.exptime, tresfib.comptime, fibkey, fib1, fib2, fib3)
	else if (ns == 1) {
	    if (ofile != "")
		print (filename, > ofile)
	    else
		outlist = filename
	    }
	else {
	    if (ofile != "")
		print (filename, >> ofile)
	    else
		outlist = outlist // "," // filename
	    }
	}

    if (dt && access (tempilist))
	delete (tempilist)

    if (outlist != "")
	print (outlist)

end

# Oct  1 2007	New program
# Oct  5 2007	Use tresfib to set FIBi keyword values
# Oct  9 2007	Use header parameter values from tresfib, not gethead
# Oct 10 2007	If speckey.db already exists, print it if verbose=yes
# Oct 11 2007	Add apfib and object keywords to use treslist for sorting
# Oct 11 2007	If not verbose and conditions set, output a list of files
# Oct 11 2007	Set speckey.db in tresdb.cl, not here
# Oct 12 2007	Update parameter list
# Oct 25 2007	Add output file for long lists meeting apfib/object condition
