#*** File tres/Src/t_preamp1.x
#*** October 18, 2007
#*** By Doug Mink

include	<mach.h>
include	<imhdr.h>
include	<imio.h>
include	<gset.h>

define  MAXCOL          4096
define  MAXLINE         4700

# T_PREAMP1 -- Fix one TRES preamp

procedure t_preamp1

int	i, j, nc, nr
pointer im, sp, inbuf, outbuf, imin, inpath, outpath
real	datafix[MAXCOL,MAXLINE]
char    hstring[SZ_LINE]
char    dstring[SZ_LINE]
char	errlin[SZ_LINE]
int	lpath
bool	verbose, debug, clgetb()
real	v0, epsilon, samp1, samp2, c1, c2, c3, bzero, bscale, s, value
long	v[IM_MAXDIM]

int	clgeti(), strlen()
int	imgs2r(), imgs2i()
int	imps2r(), imps2i()
long	clktime()
pointer	immap(), imgl2r, impnlr
errchk	immap()

begin
	call smark (sp)
	call salloc (imin,   SZ_FNAME,  TY_CHAR)
	call salloc (inpath,   SZ_PATHNAME,  TY_CHAR)
	call salloc (outpath,   SZ_PATHNAME,  TY_CHAR)
	verbose = clgetb ("verbose")
	debug = clgetb ("debug")

	# Get the image input name.
	call clgstr ("infile", Memc[imin], SZ_LINE)
	call clgstr ("indir", Memc[inpath], SZ_LINE)
	
	lpath = strlen (Memc[inpath])
	if (lpath > 0) {
	    if (Memc[inpath+lpath-1] != '/') {
		Memc[inpath+lpath] = '/'
		Memc[inpath+lpath+1] = EOS
		}
	    call strcat (Memc[imin], Memc[inpath], SZ_FNAME)
	    }
	else
	    call strcpy (Memc[imin], Memc[inpath], SZ_FNAME)

	iferr (im = immap (Memc[inpath], READ_WRITE, 0)) {
	    call sprintf (errlin, SZ_LINE, "TPREAMP1: Cannot read and write file %s")
		call pargstr (Memc[imin])
	    call printf ("%s\n")
		call pargstr (errlin)
	    call flush (STDOUT)
	    call clpstr ("error", errlin)
	    return
	    }
	else if (verbose) {
	    call printf ("TPREAMP1: %d x %d input image %s opened\n")
		call pargi (nc)
		call pargi (nr)
		call pargstr (Memc[imin])
	    call flush (STDOUT)
	    }


	# Get image dimensions
	nr = IM_LEN(im,1)
	nc = IM_LEN(im,2)
	bscale = 1.0
	call imgrpar (im, "BSCALE", bscale)
	bzero = 0.0
	call imgrpar (im, "BZERO", bzero)

	# Skip if not raw 16-bit image
	if (IM_PIXTYPE(im) != TY_SHORT && IM_PIXTYPE != TY_USHORT) {
	    call eprintf ("PREAMP1: Invalid pixel length of %d\n")
	    if (IM_PIXTYPE(im) == TY_LONG || IM_PIXTYPE(im) == TY_INT)
		call pargi (32)
	    else if (IM_PIXTYPE(im) == TY_REAL)
		call pargi (32)
	    else if (IM_PIXTYPE(im) == TY_DOUBLE)
		call pargi (64)
	    else
		call pargi (0)
	    return
	    }

	# Make correction
	v0 = 0.0
	epsilon = 62.5e-9 / (332.0e+3 * 6800.0e-12)
	samp1 = (27.0 + 75.0) / 2.0
	samp2 = (141.0 + 94.0) / 2.0
	c1 = (samp2 - samp1) * epsilon
	c2 = 1.0 - (153.0 * epsilon)
	c3 = (79.0 - 1.0) * epsilon
	do j = 1, nr {
	    inbuf = imgl2r (im, j)
	    do i = 1, nc {
		s = Memr[inbuf+i-1]
		value = c1 * v0
		s = s + value
		datafix[i,j] = s
		v0 = (v0 * c2) - (c3 * s)
		}
	    }
	
	# Put date into HISTORY
	call logtime (dstring, SZ_LINE)
	call sprintf (hstring,SZ_LINE,"Preamp fixed by TPREAMP1 %s")
	    call pargstr (dstring)
	call imastr (im, "PREAMPFX", hstring)

	if (verbose) {
	    call printf ("TPREAMP1: writing %d x %d output fixed image %s\n")
		call pargi (nc)
		call pargi (nr)
		call pargstr (imin)
	    call flush (STDOUT)
	    }

# Initialize vector for output
	call amovkl (long(1), v, IM_MAXDIM)

# Write the output file one line at a time
	do j = 1, nr {

# Get output buffer for this line of the image
	    v[2] = j
	    if (impnlr (im, outbuf, v) < nc) {
		call sprintf (errlin,SZ_LINE,"TPREAMP1: Cannot write line %d of %s")
		    call pargi (j)
		    call pargstr (imin)
		call printf ("%s\n")
		    call pargstr (errlin)
		call flush (STDOUT)
		call clpstr ("error", errlin)
		return
	    }
	    call achtrr (datafix[1,j], Memr[outbuf], nc)
	    }

# Close output image
	IM_CTIME(im) = clktime (long(0))
	IM_UPDATE(im) = YES
	if (verbose) {
	    call printf ("TPREAMP: Ready to write output image %s\n")
		call pargstr (imin)
	    call flush (STDOUT)
	    }
	call imunmap (im)
	if (verbose) {
	    call printf ("TPREAMP: Output preamp-fixed image %s written.\n")
		call pargstr (imin)
	    call flush (STDOUT)
	    }
	errlin[1] = EOS
	call clpstr ("ERROR", errlin)

	# Shutdown.
	call imunmap (im)
	call sfree (sp)
end

# Oct 18 2007	New program, based on preampfix.c
